/* Network Tracker
Copyright Ali BASEL
Ali BASEL <alib@sabanciuniv.edu>, <ali@basel.name.tr>
http://www.basel.name.tr
Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.
*/

#include "tracker.h"

extern char *optarg;

#ifdef LINUX
struct option longopts[] =
  {
    { "help", no_argument, NULL, 'h', },
    { "version", no_argument, NULL, 'v', },
    { "ip", required_argument, NULL, 'i', },
    { "uid", required_argument, NULL, 'u', },
    { "stats", no_argument, NULL, 's', },
    { NULL, 0, NULL, 0, }
  };
#endif

int main(int argc, char **argv)
{
    MYSQL mysql;
    MYSQL_RES *mysqlresult;
    MYSQL_ROW mysqlrow;
    struct ayr ayarlar;
    int num_rows, op, aranan=0, i;
    char query[500], mac[18]="", ip[18]="", uid[18]="";

    if( argc==1 ) usage();
    if( argc==2 ) strncpy(mac, argv[1], 18);
#ifdef LINUX
    while( (op=getopt_long(argc, argv, "hvV::i:u:s::", longopts, NULL)) != EOF )
#else 
	while( (op=getopt(argc, argv, "hvV::i:u:s::")) != EOF )
#endif
        switch(op) {
            case 'h':
                usage();
                break;
            case 'i':
                strncpy(ip, optarg, 18);
                aranan=1;
                break;
            case 'u':
                strncpy(uid, optarg, 18);
                aranan=2;
                break;
            case 's':
				print_statistics();
				break;
			default:
                usage();
    }
	
    // Reads sql hostname, username and password from CONFIG_FILE file
    if( !config_oku(&ayarlar) ) {
        printf("%s: Please look at syslog message file for the error !\n\n", argv[0]);
        exit(0);
    }
    
    mysql_init(&mysql);
    
    if(!mysql_real_connect(&mysql, ayarlar.sqlhost, ayarlar.sqluser, ayarlar.sqlpass, ayarlar.sqldb, SQLPORT, NULL, 0)) {
    	printf("%s: Failed to connect to database: Error: %s\n", argv[0], mysql_error(&mysql));
        return 0;
    }
    switch(aranan) {
        case 0:
            sprintf(query, "select * from macs where mac='%s'", mac);
            break;
        case 1:
            sprintf(query, "select * from macs where IP='%s'", ip);
            break;
        case 2:
            sprintf(query, "select * from macs where uid='%s'", uid);    
    }

    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        printf("%s: Error:%s, Query:%s", argv[0], mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
        if( num_rows ) {
            for(i=0; i<num_rows; i++) {
                mysqlrow = mysql_fetch_row(mysqlresult);
	            printf("\nMAC=%s, ifname=%s, SwIP=%s, IP=%s, Time=%s, uid:%s\n", mysqlrow[0], mysqlrow[1], mysqlrow[2], mysqlrow[3], mysqlrow[4], mysqlrow[5]);
            }
	        if( mysqlresult ) mysql_free_result(mysqlresult);
        }
        else printf("\nNot found in the Database!\n");    
    }


    sprintf(query, "select mac from macs");
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        printf("Error:%s, Query:%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
        printf("\nDatabase'deki MAC Sayisi:%d, ", num_rows);
    }


    sprintf(query, "select LastUpdateTime from Stats");
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        printf("Error:%s, Query:%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
        if( num_rows ) {
            mysqlrow = mysql_fetch_row(mysqlresult);
	        printf("Database LastUpdateTime: %s\n\n", mysqlrow[0]);
	        if( mysqlresult ) mysql_free_result(mysqlresult);
        }
        else printf("\nDatabase LastUpdateTime not found !\n\n");
    }
    
    mysql_close(&mysql);
	return 1;
}

void usage(void)
{
    printf("\nVersion: %s\n", VERSION);
	puts("usage: tracker mac_address    Searches the MAC address in the database");
    puts("       tracker -u uid         Searches the LDAP user ID in the database");
    puts("       tracker -i ip_address  Searches the IP address in the database");
    puts("       tracker -s             Prints statistics about MAC tracking");
	puts("       tracker -h, --help     Prints these info");
	puts("       tracker -v, --version  Prints these info\n");
	puts("tracker searches the trackerd database which is periodically updated by trackerd using SNMP.");
	puts("\nAli BASEL <alib@sabanciuniv.edu>\nhttp://www.basel.name.tr\n");
	exit(0);
}

void print_statistics(void)
{
	MYSQL mysql;
    MYSQL_RES *mysqlresult;
    MYSQL_ROW mysqlrow;
    struct ayr ayarlar;
    int num_rows;
    char query[2048];

	// Reads sql hostname, username and password from CONFIG_FILE file
    if( !config_oku(&ayarlar) ) {
        printf("Please look at syslog message file for the error !\n\n");
        exit(0);
    }
        
    mysql_init(&mysql);
    
    if(!mysql_real_connect(&mysql, ayarlar.sqlhost, ayarlar.sqluser, ayarlar.sqlpass, ayarlar.sqldb, SQLPORT, NULL, 0)) {
    	printf("Failed to connect to database: Error: %s\n", mysql_error(&mysql));
        exit(0);
    }
	

	sprintf(query, "select Status, LastUpdateTime, LastUpdateFinishTime, CurrentTotalMACNumber, PeakTotalMACNumber, PeakTotalMACNumberDate, TotalMACNumber from Stats");
    if( mysql_real_query(&mysql, query, strlen(query)) ) {
        printf("Error:%s, Query:%s", mysql_error(&mysql), query);
    }
    else { // query succeeded, process any data returned by it
        mysqlresult = mysql_store_result(&mysql);
        num_rows = mysql_num_rows(mysqlresult);
        if( num_rows ) {
            mysqlrow = mysql_fetch_row(mysqlresult);
	        printf("\n\nTracking Process is currently: %s\
			\n\nLast tracking process start time: %s\
			\n\nLast tracking process finish time: %s\
			\n\nTotal MAC address number found in the last tracking process (Current User Number in the Network): %d\
			\n\nThe max. total MAC number found in a tracking process: %d\
			\n\nThe date of the peak: %s\
			\n\nTotal unique MAC number in the database:%d\n\n\n", mysqlrow[0], mysqlrow[1], mysqlrow[2], atoi(mysqlrow[3]), atoi(mysqlrow[4]), mysqlrow[5], atoi(mysqlrow[6]));
	        if( mysqlresult ) mysql_free_result(mysqlresult);
        }
        else printf("\nThere is no record in the Database !!\n\n");
    }
    
    mysql_close(&mysql);
	
	exit(0);
}
