#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "trackerdbgui.h"
#include "interface.h"
#include "support.h"

int executeQuery(Params *params,char *query)
{   
   ldebug(2, "Executing: %s\n",query);
   if (sql_query(params->dbsock,query) < 0)
   {
      showSqlError(params, query);
      return -1;
   }
   return 0;
}

int getNumRecords(TabInfo *ti)
{
   SQL_RES *res;
   char query[MAXQUERY];
   
   ldebug(2,"getNumRecords called\n");
   
   snprintf(query,sizeof(query) - 1,SELECT,ti->table);
   
   if(executeQuery( (Params *)(ti->params), query) < 0) return 0;
   
   res = sql_store_result( ((Params *)(ti->params))->dbsock);
   if( !res ) {
      showSqlError( (Params *)(ti->params), "getNumRecords: Cannot store result");
      return 0;
   }

   ti->numrecs = sql_num_rows(res);
   
   sql_free_result(res);
   return ti->numrecs;
}

int getSortKey(TabInfo *ti)
{
   int y;
   
   ldebug(2,"getSortKey called\n");
   
   /* Find unique primary key */
   for(y=0;y<ti->numflds;y++)
   {
      if(IS_PRI_KEY(ti->fields[y].flags)&&IS_UNIQUE(ti->fields[y].flags))
      {
	 //writeLog("Using unique primary key %d for sort in %s\n",y, p->table);
	 return y;
      }
   }
   
   /* Else, find primary key */
   for(y=0; y<ti->numflds; y++)
   {
      if(IS_PRI_KEY(ti->fields[y].flags))
      {
	 //writeLog("Using primary key %d for sort in %s\n",y, p->table);
	 return y;
      }
   }
   
   /* Else, find first unique key */
   for(y=0; y<ti->numflds; y++)
   {
      if(IS_UNIQUE(ti->fields[y].flags))
      {
	 //writeLog("Using unique key %d for sort in %s\n",y, p->table);
	 return y;
      }
   }
   
   //writeLog("No unique and/or primary key in %s,using first field (0).\n", p->table);
   return 0;
}

gchar *getFieldTypeString(TabInfo *ti, int index)
{
   ldebug(2,"getFieldTypeString entered\n");
   
   switch(ti->fields[index].type)
   {
    case FIELD_TYPE_STRING:     return "String";    break;
    case FIELD_TYPE_VAR_STRING: return "VarString"; break;
    case FIELD_TYPE_FLOAT:      return "Float";     break;
    case FIELD_TYPE_DOUBLE:     return "Double";    break;
    case FIELD_TYPE_LONGLONG:
    case FIELD_TYPE_INT24:
    case FIELD_TYPE_DECIMAL:
    case FIELD_TYPE_TINY:
    case FIELD_TYPE_SHORT:
    case FIELD_TYPE_LONG:       return "Integer";   break;
    case FIELD_TYPE_DATE:       return "Date";      break;
    case FIELD_TYPE_TIME:       return "Time";      break;
    case FIELD_TYPE_TIMESTAMP:  return "TimeStamp"; break;
    case FIELD_TYPE_DATETIME:   return "Datetime";  break;
    case FIELD_TYPE_YEAR:       return "Year";      break;
    case FIELD_TYPE_TINY_BLOB:
    case FIELD_TYPE_MEDIUM_BLOB:
    case FIELD_TYPE_LONG_BLOB:
    case FIELD_TYPE_BLOB:       return "Blob:";     break;
    default:
      //writeLog("Unknown fieldtype %d detected !\n", ti->fields[index].type);
      return "Unknown"; break;
   }
}
