#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "trackerdbgui.h"
#include "interface.h"
#include "support.h"

/* Creates TabInfo struct for the given table name, and associate it with the given GtkTreeViewName 
	creates a GtkListStore and populates with the values in the table */
TabInfo *newTabInfostruct(Params *p, char *tablename, char *GtkTreeViewName)
{
	int   x, i, tlen;
	char query[MAXQUERY], buf[255];
	SQL_RES *res;
	TabInfo *ti;
	SQL_ROW row;
	GtkListStore *store;
	GtkTreeIter iter;
	GtkTreeViewColumn *column;
	GtkCellRenderer *renderer;
	GtkWidget *top=(GtkWidget *)p->mainwindow;

	ldebug(2,"newTabInfostruct entered table:%s\n", tablename);
  	ti = (TabInfo *)g_malloc(sizeof(TabInfo));
	if(ti == NULL) exit(1);
	
	if( !strlen(tablename) ) exit(1);
	ti->table=(char *)g_malloc(sizeof(char) * strlen(tablename)+1);
	strcpy(ti->table, tablename);
	   
   res = sql_list_fields( p->dbsock, ti->table, 0);
   if (res == NULL) {
      //showSqlError( p->dbsock,"newTabInfostruct: Cannot fetch fieldnames for %s", ti->table);
      free(ti->table );
      return 0;
   }
   
   ti->numflds = sql_num_fields(res); /* get number of fields */
   ldebug(2, "newTabInfostruct: numflds:%d\n", ti->numflds);
   ti->nrcols=ti->numflds;
   
   ti->fields = (SQL_FIELD *)g_malloc(sizeof(SQL_FIELD) * ti->numflds);
   ti->titles = (char **)g_malloc(sizeof(gchar *) * ti->numflds);
   ti->field_types_for_gtk_list_store=(GType *)g_malloc( sizeof(GType)*ti->numflds );
   
	for(x=0; x<ti->numflds; x++) {
		SQL_FIELD *field;
		field = sql_fetch_field(res);
		tlen = strlen(field->name) + 1;
		ldebug(2, "length of name=%d\n", tlen);
		ti->fields[x].name = (char *)g_malloc(tlen);
		strncpy(ti->fields[x].name, field->name, tlen);
		ldebug(2, "field name[%d]=%s\n", x, ti->fields[x].name);
		tlen=-1;
		if( !field->def) { /* if default value is not defined in the SQL */
			tlen = 1;
		}	
		else tlen = strlen(field->def) + 1;
		ldebug(2, "length of default value=%d\n", tlen);
		ti->fields[x].def = (char *)g_malloc(tlen);
		if( tlen==1 )ti->fields[x].def[0]=(char)NULL;
		else strncpy(ti->fields[x].def, field->def, tlen); /* field's default value */
		ldebug(2, "field default value[%d]=%s-\n", x, ti->fields[x].def);
		
		tlen = strlen(field->name) + 1;
		ti->titles[x]=(char *)g_malloc(tlen);
		strncpy(ti->titles[x], field->name, tlen);
		
		ti->fields[x].type=field->type;
		ti->fields[x].flags=field->flags;
		ti->fields[x].length=field->length;
				
		ti->field_types_for_gtk_list_store[x]=field->type;
		ldebug(2,"field[%d] MySQL type:%d\n", x, ti->field_types_for_gtk_list_store[x]);
   }
	
	
   ti->sortkey = getSortKey(ti);
   ldebug(2, "sortkey no:%d\n", ti->sortkey);
   sql_free_result(res);
	      
	ti->selectedrow  = -1;
    ti->frame = GTK_WIDGET(lookup_widget(top, "Main_Notebook"));
    ti->params=p;
    ti->mainwindow = p->mainwindow;
	ti->gtktreeview=(GtkTreeView *)GTK_WIDGET(lookup_widget(top, GtkTreeViewName));
	/* Bu ti icin yeniden gtk_list_store olusturmak gerektiginde kullanilacak */
	
	
	renderer = gtk_cell_renderer_text_new();
	/* gtktreeview icindeki gtkliststore grid'inin editable olmasi saglaniyor */
	g_object_set(renderer, "editable", TRUE, NULL);
	g_signal_connect(renderer, "edited", G_CALLBACK(on_field_edited), ti);
	
	
	/* gtktreeview'da columns header'larini tablodaki field isimleri olarak set eder */
	for(i=0; i < ti->nrcols; i++) {
		ldebug(2, "newTabInfostruct: tipler[%d]=%d\n", i, ti->field_types_for_gtk_list_store[i]);
		switch( ti->field_types_for_gtk_list_store[i] ) {
			// MySQL Field Types
			case FIELD_TYPE_TINY:
			case FIELD_TYPE_SHORT:
			case FIELD_TYPE_LONG:
			case FIELD_TYPE_INT24:
			case FIELD_TYPE_LONGLONG:
			case FIELD_TYPE_DECIMAL:
			case FIELD_TYPE_FLOAT:
			case FIELD_TYPE_DOUBLE:
				gtk_tree_view_insert_column_with_attributes(ti->gtktreeview, -1, ti->titles[i], renderer, "text",  i, NULL);
				column=gtk_tree_view_get_column(ti->gtktreeview, i);
				gtk_tree_view_column_set_sort_column_id(column, i);
				/* Daha sonra gtktreeview'da bir yere secildigi zaman
				column number'i bulmak boyle bir cift tanimlaniyor */
				g_object_set_data(G_OBJECT(column), "columnnum", GUINT_TO_POINTER(i));
				//tipler[i]=G_TYPE_INT;
				ti->field_types_for_gtk_list_store[i]=G_TYPE_STRING;
				break;
			case FIELD_TYPE_STRING:
			case FIELD_TYPE_VAR_STRING:
				gtk_tree_view_insert_column_with_attributes(ti->gtktreeview, -1, ti->titles[i], renderer, "text", i, NULL);
				column=gtk_tree_view_get_column(ti->gtktreeview, i);
				gtk_tree_view_column_set_sort_column_id(column, i);
				g_object_set_data(G_OBJECT(column), "columnnum", GUINT_TO_POINTER(i));	
				ti->field_types_for_gtk_list_store[i]=G_TYPE_STRING;
				break;
			default:
				ti->field_types_for_gtk_list_store[i]=G_TYPE_STRING;
				ldebug(2, "newTabInfostruct: BURAYA GELMEMELI !!!");
				exit(0);
  		}
	}
	/* gtktreeview'da columns header'lari olusturuldu */
   
	ldebug(2,"newTabInfostruct: showRecords initial\n");
	//p->sortkey is always defined if table != NULL 
	snprintf(query,sizeof(query) - 1,SELECT ORDERBY, ti->table);
	strcat(query,ti->fields[ti->sortkey].name);
	ldebug(2,"newTabInfostruct: query:%s-\n", query);
	if( executeQuery( p, query) ) {
		showSqlError( p,"newTabInfostruct: Query in showRecords failed !");
		return 0;
	}
	res = sql_store_result( p->dbsock);

	// gtktreeview'da gosterilmek uzere bir gtk_list_store olusturulmaya baslaniyor
	// gtk_list_store'un field'larinin sayisi ve tipleri tanimlaniyor
	store = gtk_list_store_newv(ti->numflds, ti->field_types_for_gtk_list_store);
	
	ti->nrrows=sql_num_rows(res);
	ldebug(2, "newTabInfostruct: nrrows=%d\n", ti->nrrows);
	for(x=0; x < ti->nrrows; x++) {
		if( (row = sql_fetch_row(res)) == NULL) {
			showSqlError( p,"newTabInfostruct: Error reading record %d !",x);
			break;
		}
		gtk_list_store_append(store, &iter);
		for(i=0; i < ti->nrcols; i++) {
			if( ti->field_types_for_gtk_list_store[i]==G_TYPE_INT ) {
				sprintf(buf, "%d", (int)row[i]);
				gtk_list_store_set(store, &iter, i, buf,-1);
			}
			else gtk_list_store_set(store, &iter, i, row[i],-1);
		}
	}
	
	free(res);
	
  	gtk_tree_view_set_model(ti->gtktreeview, GTK_TREE_MODEL(store));
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(ti->gtktreeview)), GTK_SELECTION_SINGLE);
  
	/* Provides column header clicking and sorting */
	gtk_tree_view_set_headers_clickable(ti->gtktreeview, TRUE);
	 
	g_object_unref(store); // destroy model automatically with view		
	return ti;
}

void set_master_detail_relation(Params *par, char *master_table_name, char *detail_table_name, char *master_field_name)
{
	int i, master_table_no=-1, detail_table_no=-1;
	
	for(i=0; i<NR_TABLES; i++) {
		if( !strcmp(par->TabInfoStructs[i]->table, master_table_name) )
			master_table_no=i;
		if( !strcmp(par->TabInfoStructs[i]->table, detail_table_name) )
			detail_table_no=i;
	}
	
	if( master_table_no==-1 || detail_table_no==-1 ) {
		ldebug(2, "Couldn't find master or detail table in the relation !\n");
		exit(1);
	}
			
	strncpy(par->TabInfoStructs[master_table_no]->master_field_name, master_field_name, sizeof(par->TabInfoStructs[master_table_no]->master_field_name));
	
	for(i=0; i < par->TabInfoStructs[master_table_no]->numflds; i++) 
		if( !strcmp(par->TabInfoStructs[master_table_no]->fields[i].name, master_field_name) ) {
			par->TabInfoStructs[master_table_no]->master_field_no=i;
			break;
		}
	
	/* Sets the index number of the detail table of that master table */
	par->TabInfoStructs[master_table_no]->DetailTableNo=detail_table_no; 
	
	/* This is the details of another table, so do not hide this gtkreeview */
	par->TabInfoStructs[detail_table_no]->DetailTableNo=-2; 
	ldebug(2, "Master field name:%s, field no=%d, detail table no=%d\n", par->TabInfoStructs[master_table_no]->master_field_name, par->TabInfoStructs[master_table_no]->master_field_no, par->TabInfoStructs[master_table_no]->DetailTableNo);
}
